import { Component } from '@angular/core';
import { APP_BASE_HREF } from '@angular/common';
import { RouterModule, Routes, Router } from '@angular/router';
import { TestBed, ComponentFixture, async } from '@angular/core/testing';
import { BlockUI, BlockUIModule } from 'ng-block-ui';
import { BlockUIRouterModule } from './block-ui-router.module';
import { BlockUIPreventNavigation } from './block-ui-prevent-navigation.service';


describe('block-ui-prevent-navigation', () => {
  @Component({
    selector: 'landing-comp',
    template: `<block-ui><div></div></block-ui>`
  })
  class LandingComp {
  }

  @Component({
    selector: 'template-comp',
    template: `<div></div>`
  })
  class TestComp {
    @BlockUI() blockUI: any;

    constructor(
      public router: Router
    ) { }
  }

  let cf: ComponentFixture<any>;
  let testCmp: TestComp;

  beforeEach(async(() => {
    const appRoutes: Routes = [
      {
        path: '', canActivateChild: [BlockUIPreventNavigation], children: [
          { path: '', component: TestComp },
          { path: 'landing-page', component: LandingComp }
        ]
      }
    ];

    TestBed.configureTestingModule({
      imports: [
        RouterModule.forRoot(appRoutes),
        BlockUIModule.forRoot(),
        BlockUIRouterModule.forRoot()
      ],
      declarations: [
        TestComp,
        LandingComp
      ],
      providers: [
        { provide: APP_BASE_HREF, useValue: '/' }
      ]
    }).compileComponents();

    cf = TestBed.createComponent(TestComp);
    cf.detectChanges();

    testCmp = cf.debugElement.componentInstance;
  }));

  it('should block navigation if blocking', () => {
    testCmp.blockUI.reset();
    testCmp.blockUI.start();

    cf.detectChanges();

    testCmp.router.navigate(['landing-page'])
      .then(canActivate => {
        expect(canActivate).toBeFalsy();
      });
  });

  it('should\'nt block if not blocking', () => {
    testCmp.blockUI.reset();

    cf.detectChanges();

    testCmp.router.navigate(['landing-page'])
      .then(canActivate => {
        expect(canActivate).toBeTruthy();
      });
  });
});
